(function($){
    $.fn.inputIndicator = function(options){
        var defaults = {
            bgPos : '30px'
        }
        
        var settings = $.extend( {}, defaults, options ),
            canvas = document.createElement('canvas'), // Canvas used to check text widths.
            context = canvas.getContext('2d'),
            iCaretPos, caret, totalWidth, cssRules, font, totalTextWidth, currentTextWidth, 
            locationPercentage, textWidthRatio, getSelectionDirection;
        
        function getCaretPosition(oField, direction){    
            // if IE
            if( document.selection ){
                oField.focus();
                var oSel = document.selection.createRange();
                oSel.moveStart('character', -oField.value.length);
                iCaretPos = oSel.text.length;
            }
            else if( oField.selectionStart || oField.selectionStart == '0' )
                iCaretPos = direction == 'left' ? oField.selectionStart : oField.selectionEnd;

            return iCaretPos || 0;
        }
        
        function initIndicator(e){  
            if( e.keyCode == 35 )
                caret = this.value.length;
            else if( e.keyCode == 36 )
                caret = 0;
            else
                caret = getCaretPosition(this, getSelectionDirection.direction);
          
            totalWidth = $(this).width();    // width of the input field
            
            cssRules = window.getComputedStyle(this);
            font = {
                size : parseInt(cssRules.getPropertyValue("font-size")),
                family : cssRules.getPropertyValue('font-family')
            };
            totalTextWidth = checkTextWidth(this.value, font);
            currentTextWidth = checkTextWidth(this.value.substring(0,caret), font); // from the beginning index until current caret position
            
            locationPercentage = Math.floor((currentTextWidth / totalTextWidth) * 100);
            textWidthRatio = (totalWidth / totalTextWidth);
                
            setStyles(this);
        }
        
        // update the visual style of the element
        function setStyles(el){
            // check the input's value overflows
            if( textWidthRatio > 1 )
                textWidthRatio = 0;
            $(el).css({    
                        'background-size' : textWidthRatio*100 + '%',
                        'background-position' : locationPercentage + '% ' + settings.bgPos
                    });
        }
        
        // check a given text lengh, acording to it's size and font family
        function checkTextWidth(text, font){
            context.font = font.size + "px " + font.family;

            if( context.fillText )
                return context.measureText(text).width;
            else if( context.mozDrawText )
                return context.mozMeasureText(text);
        }
        
        // find in which direction the user has selected the text, left or right
        getSelectionDirection = {
            direction : null,
            lastOffset : null,
            set : function(e){
                if( e.type == 'mousedown' )
                    getSelectionDirection.lastOffset = e.clientX;
                else if( e.type == 'mouseup' ){
                    getSelectionDirection.direction = e.clientX < getSelectionDirection.lastOffset ? 'left' : 'right';
                    //console.log(direction);
                }
            }
        };
        
        // make sure to initialize any input element that is inserted to the DOM after the plugin has first ran.
        $(document).bind('DOMNodeInserted', function(e){
            $(e.target).find(pluginSelector).trigger('keydown');
        });
        
        // init the whole thing and bind the events
        $(document)
            .on('mousedown mouseup keydown', this.selector, getSelectionDirection.set)
            .on('keydown focus select mouseup', this.selector, initIndicator);
            
        this.trigger('focus'); // make it show the first time
    };
})(window.Zepto || window.jQuery)